searchData={"items":[{"type":"module","title":"PerfectPower","doc":"A module for detecting perfect powers in mathematics.\n\nA perfect power is a positive integer that can be expressed as an integer power\nof another positive integer. More formally, a perfect power is a number n that\ncan be written as $n = a^b$ where a and b are integers greater than 1.","ref":"PerfectPower.html"},{"type":"module","title":"Examples - PerfectPower","doc":"iex> PerfectPower.exponential_form?(4)   # 2²\n    true\n    iex> PerfectPower.exponential_form?(8)   # 2³\n    true\n    iex> PerfectPower.exponential_form?(16)  # 2⁴ or 4²\n    true\n    iex> PerfectPower.exponential_form?(2)   # prime\n    false\n    iex> PerfectPower.exponential_form?(7)   # prime\n    false","ref":"PerfectPower.html#module-examples"},{"type":"module","title":"Mathematical Background - PerfectPower","doc":"Perfect powers include:\n- Perfect squares: 4 ($2^2$), 9 ($3^2$), 16 ($4^2$), 25 ($5^2$), etc.\n- Perfect cubes: 8 ($2^3$), 27 ($3^3$), 64 ($4^3$), 125 ($5^3$), etc.\n- Higher powers: 16 ($2^4$), 32 ($2^5$), 64 ($2^6$), 81 ($3^4$), etc.\n\nNumbers that are NOT perfect powers include:\n- Prime numbers: 2, 3, 5, 7, 11, 13, 17, 19, 23, etc.\n- Products of different primes: 6 ($2 \\times 3$), 10 ($2 \\times 5$), 14 ($2 \\times 7$), etc.\n- Other composite numbers that aren't perfect powers","ref":"PerfectPower.html#module-mathematical-background"},{"type":"module","title":"Performance - PerfectPower","doc":"The algorithm uses mathematical optimization to efficiently determine if a number\nis a perfect power by checking potential bases up to the square root of the number\nand using logarithmic bounds to limit the search space.","ref":"PerfectPower.html#module-performance"},{"type":"module","title":"Edge Cases - PerfectPower","doc":"- 0 and 1 are considered perfect powers by convention ($0 = 0^2$, $1 = 1^2$)\n- 2 and 3 are not perfect powers (they are prime numbers)","ref":"PerfectPower.html#module-edge-cases"},{"type":"function","title":"PerfectPower.exponential_form?/1","doc":"Determines if a number can be written as a perfect power.\n\nA perfect power is a number that can be expressed as $a^b$ where both a and b\nare integers greater than 1.","ref":"PerfectPower.html#exponential_form?/1"},{"type":"function","title":"Parameters - PerfectPower.exponential_form?/1","doc":"- `n` - A non-negative integer to check","ref":"PerfectPower.html#exponential_form?/1-parameters"},{"type":"function","title":"Returns - PerfectPower.exponential_form?/1","doc":"- `true` if n is a perfect power\n  - `false` if n is not a perfect power","ref":"PerfectPower.html#exponential_form?/1-returns"},{"type":"function","title":"Examples - PerfectPower.exponential_form?/1","doc":"iex> PerfectPower.exponential_form?(0)   # 0¹\n    true\n    iex> PerfectPower.exponential_form?(1)   # 1¹\n    true\n    iex> PerfectPower.exponential_form?(2)   # prime\n    false\n    iex> PerfectPower.exponential_form?(3)   # prime\n    false\n    iex> PerfectPower.exponential_form?(4)   # 2²\n    true\n    iex> PerfectPower.exponential_form?(8)   # 2³\n    true\n    iex> PerfectPower.exponential_form?(16)  # 2⁴ or 4²\n    true\n    iex> PerfectPower.exponential_form?(27)  # 3³\n    true\n    iex> PerfectPower.exponential_form?(100) # 10²\n    true\n    iex> PerfectPower.exponential_form?(101) # prime\n    false","ref":"PerfectPower.html#exponential_form?/1-examples"},{"type":"function","title":"Algorithm - PerfectPower.exponential_form?/1","doc":"The function uses an optimized approach:\n\n1. Handle edge cases (0, 1, 2, 3) with explicit pattern matching\n2. For other numbers, check potential bases from 2 up to the square root\n3. For each base, calculate the corresponding exponent using logarithms\n4. Verify if the calculated exponent produces the original number\n\nThis approach is efficient because:\n- It limits the search space using mathematical bounds\n- It uses logarithmic calculations to find potential exponents\n- It avoids brute force checking of all possible combinations","ref":"PerfectPower.html#exponential_form?/1-algorithm"},{"type":"function","title":"Mathematical Properties - PerfectPower.exponential_form?/1","doc":"- If $n = a^b$, then $b \\leq \\log_2(n)$\n- The base $a$ must be $\\leq \\sqrt{n}$ for $b \\geq 2$\n- This allows us to bound our search space efficiently","ref":"PerfectPower.html#exponential_form?/1-mathematical-properties"},{"type":"extras","title":"PerfectPower","doc":"# PerfectPower\n\nA mathematical library for detecting perfect powers in Elixir.\n\nA perfect power is a positive integer that can be expressed as an integer power of another positive integer. More formally, a perfect power is a number n that can be written as $n = a^b$ where a and b are integers greater than 1.","ref":"readme.html"},{"type":"extras","title":"Installation - PerfectPower","doc":"The package can be installed by adding `perfect_power` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:perfect_power, \"~> 1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - PerfectPower","doc":"```elixir\n# Check if a number is a perfect power\nPerfectPower.exponential_form?(4)   # true  (2^2)\nPerfectPower.exponential_form?(8)   # true  (2^3)\nPerfectPower.exponential_form?(16)  # true  (2^4 or 4^2)\nPerfectPower.exponential_form?(27)  # true  (3^3)\nPerfectPower.exponential_form?(100) # true  (10^2)\n\n# Numbers that are not perfect powers\nPerfectPower.exponential_form?(2)   # false (prime)\nPerfectPower.exponential_form?(7)   # false (prime)\nPerfectPower.exponential_form?(6)   # false (2×3)\nPerfectPower.exponential_form?(10)  # false (2×5)\n```","ref":"readme.html#usage"},{"type":"extras","title":"Mathematical Background - PerfectPower","doc":"","ref":"readme.html#mathematical-background"},{"type":"extras","title":"What are Perfect Powers? - PerfectPower","doc":"Perfect powers are numbers that can be written as $a^b$ where both a and b are integers greater than 1. They include:\n\n- **Perfect squares**: 4 ($2^2$), 9 ($3^2$), 16 ($4^2$), 25 ($5^2$), 36 ($6^2$), 49 ($7^2$), 64 ($8^2$), 81 ($9^2$), 100 ($10^2$)\n- **Perfect cubes**: 8 ($2^3$), 27 ($3^3$), 64 ($4^3$), 125 ($5^3$), 216 ($6^3$), 343 ($7^3$), 512 ($8^3$), 729 ($9^3$), 1000 ($10^3$)\n- **Higher powers**: 16 ($2^4$), 32 ($2^5$), 64 ($2^6$), 128 ($2^7$), 256 ($2^8$), 512 ($2^9$), 1024 ($2^{10}$), 81 ($3^4$), 243 ($3^5$), 729 ($3^6$), 625 ($5^4$), 3125 ($5^5$)","ref":"readme.html#what-are-perfect-powers"},{"type":"extras","title":"Numbers that are NOT Perfect Powers - PerfectPower","doc":"- **Prime numbers**: 2, 3, 5, 7, 11, ...\n- **Products of different primes**: 6 ($2 \\times 3$), 10 ($2 \\times 5$), 14 ($2 \\times 7$), 15 ($3 \\times 5$), ...","ref":"readme.html#numbers-that-are-not-perfect-powers"},{"type":"extras","title":"Edge Cases - PerfectPower","doc":"- **0 and 1**: By convention, 0 and 1 are considered perfect powers (0 = $0^2$, 1 = $1^2$)","ref":"readme.html#edge-cases"},{"type":"extras","title":"Algorithm - PerfectPower","doc":"The library uses an optimized algorithm to efficiently determine if a number is a perfect power:\n\n1. **Edge case handling**: 0, 1, 2, and 3 are handled with explicit pattern matching\n2. **Mathematical bounds**: For other numbers, the algorithm checks potential bases from 2 up to the square root of the number\n3. **Logarithmic calculation**: For each potential base, it calculates the corresponding exponent using logarithms\n4. **Verification**: It verifies if the calculated exponent produces the original number","ref":"readme.html#algorithm"},{"type":"extras","title":"Performance Characteristics - PerfectPower","doc":"- **Time complexity**: $O(\\log n)$ for most numbers\n- **Space complexity**: $O(1)$\n- **Mathematical optimization**: Uses logarithmic bounds to limit search space\n- **Efficient for large numbers**: Can handle numbers up to $2^512$ and beyond","ref":"readme.html#performance-characteristics"},{"type":"extras","title":"Examples - PerfectPower","doc":"","ref":"readme.html#examples"},{"type":"extras","title":"Perfect Powers - PerfectPower","doc":"```elixir\n# Perfect squares\nPerfectPower.exponential_form?(4)   # true  (2²)\nPerfectPower.exponential_form?(9)   # true  (3²)\nPerfectPower.exponential_form?(16)  # true  (4²)\n\n# Perfect cubes\nPerfectPower.exponential_form?(8)   # true  (2³)\nPerfectPower.exponential_form?(27)  # true  (3³)\nPerfectPower.exponential_form?(64)  # true  (4³)\n\n# Higher powers\nPerfectPower.exponential_form?(16)  # true  (2⁴)\nPerfectPower.exponential_form?(32)  # true  (2⁵)\nPerfectPower.exponential_form?(64)  # true  (2⁶)\nPerfectPower.exponential_form?(128) # true  (2⁷)\n```","ref":"readme.html#perfect-powers"},{"type":"extras","title":"Non-Perfect Powers - PerfectPower","doc":"```elixir\n# Prime numbers\nPerfectPower.exponential_form?(2)   # false\nPerfectPower.exponential_form?(3)   # false\nPerfectPower.exponential_form?(5)   # false\n\n# Composite numbers that are not perfect powers\nPerfectPower.exponential_form?(6)   # false (2×3)\nPerfectPower.exponential_form?(10)  # false (2×5)\nPerfectPower.exponential_form?(14)  # false (2×7)\n```","ref":"readme.html#non-perfect-powers"},{"type":"extras","title":"Large Numbers - PerfectPower","doc":"```elixir\n# Huge perfect powers\nPerfectPower.exponential_form?(Bitwise.bsl(1, 256)) # true  (2²⁵⁶)\nPerfectPower.exponential_form?(Bitwise.bsl(1, 512)) # true  (2⁵¹²)\n```","ref":"readme.html#large-numbers"},{"type":"extras","title":"Testing - PerfectPower","doc":"Run the test suite:\n\n```bash\nmix test\n```\n\nThe test suite includes comprehensive coverage of:\n- Perfect squares, cubes, and higher powers\n- Prime numbers (up to 997)\n- Composite non-perfect powers\n- Edge cases (0, 1, 2, 3)\n- Large numbers for performance testing","ref":"readme.html#testing"},{"type":"extras","title":"Documentation - PerfectPower","doc":"Documentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc):\n\n```bash\nmix docs\n```","ref":"readme.html#documentation"},{"type":"extras","title":"License - PerfectPower","doc":"Copyright (c) 2025 University of Kitakyushu\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at [http://www.apache.org/licenses/LICENSE-2.0](http://www.apache.org/licenses/LICENSE-2.0)\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"readme.html#license"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright [yyyy] [name of copyright owner]\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [1.0.0] - 2025-07-21","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Initial implementation of `PerfectPower.exponential_form?/1` function\n- Mathematical algorithm for detecting perfect powers\n- Edge case handling for 0, 1, 2, and 3\n- Type specifications for function parameters and return values\n- Comprehensive documentation with mathematical background\n- Project configuration with development tooling\n- Apache 2.0 license\n- Development dependencies for code quality and documentation\n- Comprehensive test suite with 386 test cases\n- Support for huge numbers (2^256, 2^512)\n- Mathematical documentation and examples\n- Performance optimizations for large numbers","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Enhanced module documentation with detailed mathematical background\n- Improved README with comprehensive usage examples\n- Added type specifications for better code clarity","ref":"changelog.html#changed"},{"type":"extras","title":"Features - Changelog","doc":"- Efficient perfect power detection using logarithmic bounds\n- Support for all perfect powers: squares, cubes, and higher powers\n- Mathematical optimization for large numbers\n- Comprehensive test coverage including edge cases and large numbers","ref":"changelog.html#features"},{"type":"extras","title":"Technical Details - Changelog","doc":"- Algorithm time complexity: O(log n) for most numbers\n- Space complexity: O(1)\n- Uses mathematical bounds to limit search space\n- Handles numbers up to 2^512 and beyond efficiently","ref":"changelog.html#technical-details"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}